import sys
import subprocess
import pkg_resources

required =  {"numpy", "regex", "pillow", "panda", "shortuuid", "aggdraw", "psutil", "scipy", "pyspectator", "pyperclip", "matplotlib", "requests", "pyspellchecker", "speechrecognition", "pytesseract", "pytz", "shortuuid"}
installed = {pkg.key for pkg in pkg_resources.working_set}
missing = required - installed

if missing:
    python = sys.executable
    subprocess.check_call([python, '-m', 'pip', 'install', *missing], stdout=subprocess.DEVNULL)


import string
import random

def random_integer(n):
    characters = string.digits
    password = ''.join(random.choice(characters) for i in range(n))
    return password

def random_string(n):
    characters = string.ascii_uppercase
    password = ''.join(random.choice(characters) for i in range(n))
    return str(password)

def random_serial(n):
    characters = string.digits + string.ascii_uppercase
    password = ''.join(random.choice(characters) for i in range(n))
    return str(password)


from PIL import Image
import aggdraw

transparent = (0, 0, 0, 0)
black = (0, 0, 0)
white = (255, 255, 255)
red = (255, 0, 0)
blue = (0, 0, 255)
fuchsia = (255, 0, 255)
green = (0, 255, 0)
aqua = (0, 255, 255)
yellow = (255, 255, 0)

def add_padding(image_path, padding_value, color):
    image = Image.open(image_path)
    top = right = bottom = left = padding_value
    width, height = image.size
    new_width = width + right + left
    new_height = height + top + bottom
    result = Image.new(image.mode, (new_width, new_height), color)
    result.paste(image, (left, top))
    result.save(image_path)
    
def round_corners(image_path, radius):
    image = Image.open(image_path)
    mask = Image.new('L', image.size)
    draw = aggdraw.Draw(mask)
    brush = aggdraw.Brush('white')
    width, height = mask.size
    draw.pieslice((0, 0, radius*2, radius*2), 90, 180, 255, brush)
    draw.pieslice((width - radius*2, 0, width, radius*2), 0, 90, None, brush)
    draw.pieslice((0, height - radius * 2, radius *
                   2, height), 180, 270, None, brush)
    draw.pieslice((width - radius * 2, height - radius *
                   2, width, height), 270, 360, None, brush)
    draw.rectangle((radius, radius, width - radius, height - radius), brush)
    draw.rectangle((radius, 0, width - radius, radius), brush)
    draw.rectangle((0, radius, radius, height-radius), brush)
    draw.rectangle((radius, height-radius, width-radius, height), brush)
    draw.rectangle((width-radius, radius, width, height-radius), brush)
    draw.flush()
    image = image.convert('RGBA')
    image.putalpha(mask)
    image.save(image_path)

def turn_to_square(image_path, color):
    image = Image.open(image_path)
    x, y = image.size
    size = max(256, x, y)
    new_image = Image.new('RGB', (size, size), color = color)
    new_image.paste(image, (int((size - x) / 2), int((size - y) / 2)))
    new_image.save(image_path)



from datetime import datetime

months = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" ]

def get_date():
    from datetime import date
    today = date.today()
    d = today.strftime("%m/%d/%Y")
    return d

def get_month():
    month_index = datetime.now().month - 1
    return [month_index, months[month_index]]
    
def get_year():
    return int(datetime.now().year)


import os
import shutil 

def read_lines(file_path):
    with open(file_path) as f:
            lines = f.readlines()
    return lines

def count_lines(file_path):
    return len(read_lines(file_path))

def count_folders(folder_path):
    return len(list_directories(folder_path))

def count_files(folder_path):
    return len(list_files(folder_path))

def get_file_info(file_path):
    split_path = os.path.splitext(file_path)
    file_name = file_path.split("/")[-1]
    file_extension = split_path[1]
    file_location = file_path.split("/" + file_name)[0]
    return [file_name, file_extension, file_location]

def get_file_name(file_path):
    return get_file_info(file_path)[0]

def get_file_extension(file_path):
    return get_file_info(file_path)[1]

def get_file_location(file_path):
    return get_file_info(file_path)[2]

def is_file(path):
    return os.path.isfile(path)
          
def is_folder(path):
    return os.path.isdir(path)

def is_existing(path):
    return os.path.exists(path) 

def list_directories(folder_path):
    return [
        d for d in (os.path.join(folder_path, d1) for d1 in os.listdir(folder_path))
        if os.path.isdir(d)
    ]

def list_files(folder_path):
    return [f for f in os.listdir(folder_path) if os.path.isfile(os.path.join(folder_path, f)) if f[0] != "."]

def is_empty(folder_path):
    return len(os.listdir(folder_path)) == 0

def delete_file(file_path):
    os.remove(file_path)

def delete_folder(folder_path):
    shutil.rmtree(folder_path)


from compress.installer import *
from compress.functions.main import *
from compress.functions.manage import *
from compress.functions.random import *
from compress.functions.time import *
from compress.functions.image import *


import subprocess, platform, string
from datetime import datetime
from time import sleep

def has_duplicates(lst):
    if len(lst) == len(set(lst)):
        return False
    else:
        return True

def estimateReadingtime(words_number):
    reading = (0.4615/60) * words_number
    return reading

def print_options(array):
    for index, option in enumerate(array):
        print(str(index+1) + ".", option)

def first_chars(str, chars):
    return(str[:chars])

def last_chars(str, chars):
    return (str[-chars:])

def reverse_string(str):
    return str[::-1]

def clear_terminal():
    if platform.system()=="Windows":
        subprocess.Popen("cls", shell=True).communicate() 
    else:
        print("\033c", end="")

def number_of_words(title):
    text = title.split(" ")
    total_words = len(text)
    return total_words

def hex_to_rgb(hex):
    rgb = []
    for i in (0, 2, 4):
        decimal = int(hex[i:i+2], 16)
        rgb.append(decimal)
    return tuple(rgb)